/*
 *  Authors:  tiandongdong <tiandongdong@kylinos.cn>
 *  Authors:  litianhao <litianhao@kylinos.cn>
 *
 *  Copyright (c) KylinSoft Co., Ltd [2024].All rights reserved.
 */
#pragma once

#define KYSEC_SCENE_USB_SERIAL_LEN 256
#define KYSEC_SCENE_USB_NAME_LEN 256
#define KYSEC_SCENE_NETCARD_NAME_LEN 128

enum kysec_scene_devctl_type {
  KYSEC_SCENE_DEVCTL_INF = 1,
  KYSEC_SCENE_DEVCTL_DEV,
  KYSEC_SCENE_DEVCTL_USB,
};

enum kysec_scene_inf_type {
  KYSEC_SCENE_DEVCTL_INF_SERIAL = 1,
  KYSEC_SCENE_DEVCTL_INF_PARALLEL,
  KYSEC_SCENE_DEVCTL_INF_1394,
  KYSEC_SCENE_DEVCTL_INF_PCMCIA,
  KYSEC_SCENE_DEVCTL_INF_TYPEC,
  KYSEC_SCENE_DEVCTL_INF_HDMI,
  KYSEC_SCENE_DEVCTL_INF_DISPLAYPORT,
  KYSEC_SCENE_DEVCTL_INF_SCANNER,
  KYSEC_SCENE_DEVCTL_INF_PCIE,
  KYSEC_SCENE_DEVCTL_INF_SATA,
  KYSEC_SCENE_DEVCTL_INF_MMC,
};

enum kysec_scene_dev_type {
  KYSEC_SCENE_DEVCTL_DEV_BLUETOOTH = 1,
  KYSEC_SCENE_DEVCTL_DEV_INFRARED,
  KYSEC_SCENE_DEVCTL_DEV_MODEM,
  KYSEC_SCENE_DEVCTL_DEV_STORAGE_DRIVE,
  KYSEC_SCENE_DEVCTL_DEV_SCSI_RAID,
  KYSEC_SCENE_DEVCTL_DEV_WINCE,
  KYSEC_SCENE_DEVCTL_DEV_FLOPPY_DRIVE,
  KYSEC_SCENE_DEVCTL_DEV_IMAGE,
  KYSEC_SCENE_DEVCTL_DEV_MULTI_SERIAL_CARD,
  KYSEC_SCENE_DEVCTL_DEV_ETHERNET,
  KYSEC_SCENE_DEVCTL_DEV_WIRELESS,
  KYSEC_SCENE_DEVCTL_DEV_PRINTER,
  KYSEC_SCENE_DEVCTL_DEV_PRINTER_LOCAL,
  KYSEC_SCENE_DEVCTL_DEV_PRINTER_NET,
  KYSEC_SCENE_DEVCTL_DEV_CDROM,
  KYSEC_SCENE_DEVCTL_DEV_VOLUMESHADOW,
  KYSEC_SCENE_DEVCTL_DEV_TAPE,
};

enum kysec_scene_usb_type {
  KYSEC_SCENE_DEVCTL_USB_ALL = 0,
  KYSEC_SCENE_DEVCTL_USB_PER_INTERFACE = 1,
  KYSEC_SCENE_DEVCTL_USB_AUDIO,
  KYSEC_SCENE_DEVCTL_USB_COMM,
  KYSEC_SCENE_DEVCTL_USB_HID,
  KYSEC_SCENE_DEVCTL_USB_PHYSICAL,
  KYSEC_SCENE_DEVCTL_USB_STILL_IMAGE,
  KYSEC_SCENE_DEVCTL_USB_PRINTER,
  KYSEC_SCENE_DEVCTL_USB_MASS_STORAGE,
  KYSEC_SCENE_DEVCTL_USB_HUB,
  KYSEC_SCENE_DEVCTL_USB_CDC_DATA,
  KYSEC_SCENE_DEVCTL_USB_CSCID,
  KYSEC_SCENE_DEVCTL_USB_CONTENT_SEC,
  KYSEC_SCENE_DEVCTL_USB_VIDEO,
  KYSEC_SCENE_DEVCTL_USB_WIRELESS_CONTROLLER,
  KYSEC_SCENE_DEVCTL_USB_MISC,
  KYSEC_SCENE_DEVCTL_USB_APP_SPEC,
  KYSEC_SCENE_DEVCTL_USB_VENDOR_SPEC,
  KYSEC_SCENE_DEVCTL_USB_PERSONAL_HEALTH,
  KYSEC_SCENE_DEVCTL_USB_DIAGNOSTIC,
  /**
   * @brief 外置光驱
   * @detail mass_storage 8020
   */
  KYSEC_SCENE_DEVCTL_USB_8020,
};

enum kysec_scene_devctl_perm {
  /**
   * @brief 启用设备
   */
  KYSEC_SCENE_DEVCTL_ENABLE = 1,

  /**
   * @brief 禁用设备
   */
  KYSEC_SCENE_DEVCTL_DISABLE = 2,

  /**
   * @brief 设备处于只读状态
   * @detail 仅对部分设备类型有效，如cdrom
   */
  KYSEC_SCENE_DEVCTL_RO = 5,
};

struct kysec_scene_devctl_type_policy {
  /**
   * enum kysec_scene_devctl_type
   */
  int devctl_type;
  int sub_type;
  /**
   * enum kysec_scene_devctl_perm
   */
  int perm;
};

struct kysec_scene_devctl_pvid_policy {
  /**
   * @brief 指定usb设备的类型，usb_type
   * @detail enum kysec_scene_usb_type
   */
  int usb_type;

  /**
   * @brief 指定usb设备的product id
   */
  unsigned int pid;

  /**
   * @brief 指定usb设备的vendor id
   */
  unsigned int vid;

  /**
   * @brief 序列号，最长长度为`KYSEC_SCENE_USB_SERIAL_LEN`
   * @detail 可选字段。下发时可指定序列号，也可以不指定（此时要填\0）
   */
  char serial[KYSEC_SCENE_USB_SERIAL_LEN];

  /**
   * @brief 指定权限
   * @detail enum kysec_scene_devctl_perm
   */
  int perm;

  /**
   * @brief 设备名称
   */
  char name[KYSEC_SCENE_USB_NAME_LEN];
};

/**
 * @brief sys中device_info内记录的设备接入状态信息
 */
struct kysec_scene_sys_device_info {
  /**
   * @brief usb类型
   */
  int usb_type;

  /**
   * @brief product id
   */
  unsigned int pid;

  /**
   * @brief vendor id
   */
  unsigned int vid;

  /**
   * @brief 权限
   * @detail 1: 允许, 2: 禁止, 5: 只读
   */
  int perm;

  /**
   * @brief 产品名
   */
  char product[128];

  /**
   * @brief 制造商名
   */
  char manufacturer[128];

  /**
   * @brief 序列号
   */
  char serial[128];

  /**
   * @brief 总线上的设备名
   */
  char device_name[30];

  /**
   * @brief 展示用的名字
   */
  char display_name[256];
};

enum kysec_scene_devctl_netcard_type {
  KYSEC_SCENE_DEVCTL_WIRELESS_SSID = 1,
  KYSEC_SCENE_DEVCTL_NETCARD,
};

struct kysec_scene_devctl_netcard_policy {
  char name[KYSEC_SCENE_NETCARD_NAME_LEN];
  /**
   * enum kysec_scene_devctl_netcard_type
   */
  int netcard_type;
};

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief 根据设备类型添加管控策略
 * @detail 可添加inf/dev/usb这三大类策略
 * @param policies[in] 添加的策略
 * @param count[in] 添加的策略的数量
 * @return 0:成功 其他:失败
 */
int kysec_scene_add_devctl_by_type(
    const struct kysec_scene_devctl_type_policy *policies, int count);

/**
 * @brief 根据设备类型删除管控策略
 * @detail 可删除inf/dev/usb这三大类策略
 * @param policies[in] 添加的策略
 * @param count[in] 添加的策略的数量
 * @return 0:成功 其他:失败
 */
int kysec_scene_remove_devctl_by_type(
    const struct kysec_scene_devctl_type_policy *policies, int count);

/**
 * @brief 获取所有根据设备类型下发的策略
 * @param count[out] 策略数量
 * @return 策略数组。非null：成功 null：失败
 *        调用者需要使用free释放返回值
 */
struct kysec_scene_devctl_type_policy *
kysec_scene_get_devctl_all_types(int *count);

/**
 * @brief 根据某一个设备类型获取该类型的所有管控策略
 * @detail 获取某一个类型的策略
 * @param type[in] 要获取哪一种设备类型的策略 enum kysec_scene_devctl_type
 * @param count[out] 策略数量
 * @return 策略数组。非null：成功 null：失败
 *        调用者需要使用free释放返回值
 */
struct kysec_scene_devctl_type_policy *
kysec_scene_get_devctl_by_type(int type, int *count);

/**
 * @brief 根据usb设备的pid，vid和序列号添加管控策略
 * @detail 添加的是针对具体usb设备的策略
 * @param policies[in] 添加的策略，包含usb类型，pid，vid和序列号
 * @param count[in] 添加的策略的数量
 * @return >=0:添加成功的策略的数量 其他:失败
 */
int kysec_scene_add_devctl_by_pvid(
    const struct kysec_scene_devctl_pvid_policy *policies, int count);

/**
 * @brief 根据usb设备的pid，vid和序列号删除管控策略
 * @detail 删除的是针对具体usb设备的策略
 *         不匹配策略的perm字段，只要设备对应上了就会删除
 * @return 0:成功 其他:失败
 * @param policies[in] 删除的策略，包含usb类型，pid，vid和序列号
 * @param count[in] 删除的策略的数量
 * @brief 删除指定设备的管控策略
 */
int kysec_scene_remove_devctl_by_pvid(
    const struct kysec_scene_devctl_pvid_policy *policies, int count);

/**
 * @brief 获取所有根据usb设备的pid，vid和序列号添加的管控策略
 * @param count[out] 策略数量
 * @return 策略数组。非null：成功 null：失败
 *        调用者需要使用free释放返回值
 */
struct kysec_scene_devctl_pvid_policy *
kysec_scene_get_devctl_all_pvid(int *count);

/**
 * @brief 根据网卡名添加管控策略
 * @param policies[in] 添加的策略
 * @param count[in] 添加的策略的数量
 * @return 0:成功 其他:失败
 */
int kysec_scene_add_devctl_by_netcard_name(
    const struct kysec_scene_devctl_netcard_policy *policies, int count);

/**
 * @brief 根据网卡名删除管控策略
 * @param policies[in] 删除的策略
 * @param count[in] 删除的策略的数量
 * @return 0:成功 其他:失败
 */
int kysec_scene_remove_devctl_by_netcard_name(
    const struct kysec_scene_devctl_netcard_policy *policies, int count);

/**
 * @brief 获取所有根据网卡名添加的管控策略
 * @param count[out] 策略数量
 * @return 策略数组。非null：成功 null：失败
 *        调用者需要使用free释放返回值
 */
struct kysec_scene_devctl_netcard_policy *
kysec_scene_get_devctl_netcard(int *count);

/**
 * @brief 读取sys中展示的已接入设备的设备信息
 * @param count[out] 设备的数量
 * @return 设备属组。非null：成功 null：失败
 *        调用者需要使用free释放返回值
 */
struct kysec_scene_sys_device_info *
kysec_scene_read_sys_device_info(int *count);

/**
 * @brief 检查某种设备类型是否已被禁用
 * @param devctl_type[in] 大类，取enum kysec_scene_devctl_type的值
 * @param sub_type[in] 具体的设备类型，根据devctl_type取值的不同，分别
 *        取kysec_scene_inf_type，kysec_scene_dev_type，和kysec_scene_usb_type
 * @return 0：未禁用，1：已禁用，其他：失败
 */
int kysec_scene_devctl_is_type_disabled(int devctl_type, int sub_type);

/**
 * @brief 根据usb设备的pvid检查是否该设备已被禁用
 * @param usb_type[in] usb设备的类型，取enum kysec_scene_usb_type
 * @param pid[in] 设备的product id
 * @param vid[in] 设备的vendor id
 * @param serial[in] 设备的序列号
 * @return 0：未禁用，1：已禁用，其他：失败
 */
int kysec_scene_devctl_is_pvid_disabled(int usb_type, unsigned int pid,
                                        unsigned int vid, const char *serial);

/**
 * @brief 根据网卡名检查某个网卡是否已被禁用
 * @param netcard_type[in] 网卡类型，取enum kysec_scene_devctl_netcard_type
 * @param name[in] 网卡名
 * @return 0：未禁用，1：已禁用，其他：失败
 */
int kysec_scene_devctl_is_netcard_disabled(int netcard_type, const char *name);

/**
 * @brief 根据usb设备的pvid检查该设备的管控状态;
 * @param usb_type[in] usb设备的类型，取enum kysec_scene_usb_type
 * @param pid[in] 设备的product id
 * @param vid[in] 设备的vendor id
 * @param serial[in] 设备的序列号
 * @return 检查成功时:
 *         1. 如果没有管控策略，返回0
 *         2. 如果有管控策略，返回 enum kysec_scene_devctl_perm
 * 表示当前的设备权限
 *         3. 如果检查的时候失败了，返回enum KYSEC_SCENE_CALL_RESULT中的负数值
 */
int kysec_scene_devctl_get_pvid_perm(int usb_type, unsigned int pid,
                                     unsigned int vid, const char *serial);

#ifdef __cplusplus
}
#endif
