/*
 *  Authors:  tiandongdong <tiandongdong@kylinos.cn>
 *  Authors:  litianhao <litianhao@kylinos.cn>
 *
 *  Copyright (c) KylinSoft Co., Ltd [2024].All rights reserved.
 */
#pragma once

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief 根据ko文件绝对路径添加内核模块防卸载保护
 * @param path 内核模块文件绝对路径或内核模块名称
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_kmodctl_add_by_path(const char *path);

/**
 * @brief 根据ko文件绝对路径删除内核模块防卸载保护
 * @param path 内核模块文件绝对路径或内核模块名称
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_kmodctl_remove_by_path(const char *path);

/**
 * @brief 根据ko名称添加内核模块防卸载保护
 * @param name 内核模块名称
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_kmodctl_add_by_name(const char *name);

/**
 * @brief 根据ko名称删除内核模块防卸载保护
 * @param name 内核模块名称
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_kmodctl_remove_by_name(const char *name);

/**
 * @brief 获取内核模块保护策略
 * @param count 个数
 * @return 字符串数组  NULL:无数据
 *         调用者使用完需要手动释放内存
 */
char **kysec_scene_kmodctl_get_policy(int *count);

/**
 * @brief 通过ko路径获取模块名称
 * @param ko_path  ko文件绝对路径
 * @return 成功：字符串 NULL:失败
 */
char *kysec_scene_get_module_name_from_ko_path(const char *ko_path);

#ifdef __cplusplus
}
#endif
